package cn.com.qmth.stmms.ms.admin.api;

import cn.com.qmth.stmms.ms.admin.service.DataUploadService;
import cn.com.qmth.stmms.ms.commons.utils.specification.PagingAndSortingDTO;
import cn.com.qmth.stmms.ms.core.domain.Student;
import cn.com.qmth.stmms.ms.core.repository.PaperRepo;
import cn.com.qmth.stmms.ms.core.repository.StudentRepo;
import cn.com.qmth.stmms.ms.core.specification.StudentSpecification;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

/**
 * 考生数据api
 */
@RestController
@RequestMapping("api/students")
public class StudentApi {

    @Autowired
    private DataUploadService dataUploadService;

    @Autowired
    private StudentRepo studentRepo;

    @Autowired
    private PaperRepo paperRepo;

    /**
     * 考生列表
     *
     * @param params
     * @return
     */
    @RequestMapping(method = RequestMethod.GET)
    public PagingAndSortingDTO list(StudentSpecification params) {
        Page<Student> result = studentRepo.findAll(params.getSpecification(), params.getPageRequest());
        return new PagingAndSortingDTO(result.getNumber(), result.getSize(), result.getTotalPages(),
                result.getTotalElements(), result.getContent());
    }

    /**
     * 单个考生
     *
     * @param domain 考生id
     * @return
     */
    @RequestMapping(value = "{domain}", method = RequestMethod.GET)
    public Student getOne(@PathVariable Student domain) {
        return domain;
    }

    /**
     * 删除
     *
     * @param domain
     */
    @RequestMapping(value = "{domain}", method = RequestMethod.DELETE)
    public void remove(@PathVariable Student domain) {
        Long isExist = paperRepo.countByWorkIdAndExamNumber(domain.getWorkId(), domain.getExamNumber());
        if (isExist > 0) {
            throw new RuntimeException("该考生已上传，无法删除");
        }
        studentRepo.delete(domain);
    }

    /**
     * 修改信息
     *
     * @param domain
     * @param student
     */
    @RequestMapping(value = "{domain}", method = RequestMethod.PUT)
    public void update(@PathVariable Student domain, @RequestBody Student student) {
        Long isExist = paperRepo.countByWorkIdAndExamNumber(domain.getWorkId(), domain.getExamNumber());
        if (isExist > 0) {
            throw new RuntimeException("该考生已上传，无法修改");
        }
        domain.setName(student.getName());
        domain.setExamRoom(student.getExamRoom());
        domain.setAreaCode(student.getAreaCode());
        studentRepo.save(domain);
    }

    /**
     * 新增
     *
     * @param domain
     * @return
     */
    @RequestMapping(method = RequestMethod.POST)
    public Student create(@RequestBody Student domain) throws Exception {
        try {
            return studentRepo.save(domain);
        } catch (Exception e) {
            if (e instanceof DataIntegrityViolationException) {
                throw new Exception("修改失败，同一个考区不允许有重复的考号");
            }
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 清空所有
     *
     * @param workId 评卷工作id
     */
    @RequestMapping(method = RequestMethod.DELETE)
    public void clear(@RequestParam Long workId) {
        dataUploadService.clearStudents(workId);
    }

    /**
     * 新增
     *
     * @param studentId
     * @param subject
     * @param isManual
     * @param file
     * @throws IOException
     */
    @RequestMapping(value = "{studentId}", method = RequestMethod.POST)
    public void addPaper(@PathVariable Long studentId,
                         @RequestParam Subject subject,
                         @RequestParam(required = false) boolean isManual,
                         @RequestParam MultipartFile file)
            throws Exception {
        dataUploadService.uploadPaper(studentId, subject, file.getInputStream(), isManual);
    }

}
