package cn.com.qmth.stmms.ms.marking.api;

import cn.com.qmth.stmms.ms.core.domain.Level;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.user.MarkRight;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.MarkerGroup;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.marking.assembler.LevelStatAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.QuestionStatAssembler;
import cn.com.qmth.stmms.ms.marking.dto.LevelStatDTO;
import cn.com.qmth.stmms.ms.marking.dto.LevleProgressDTO;
import cn.com.qmth.stmms.ms.marking.dto.QuestionStatDTO;
import cn.com.qmth.stmms.ms.marking.service.StageControlService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 评卷科目数据api
 * Created by zhengmin on 2016/10/10.
 */
@RestController
@RequestMapping("api/marksubjects")
public class MarkSubjectApi {

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    @Autowired
    private QuestionStatAssembler questionStatAssembler;

    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    private LevelStatAssembler levelStatAssembler;

    @Autowired
    private MarkerGroupRepo markerGroupRepo;

    @Autowired
    private MarkUserRepo markUserRepo;

    @Autowired
    private StageControlService stageControlService;

    /**
     * 查询所有科目状态
     *
     * @param workId
     * @return
     */
    @RequestMapping(value = "getAllSubjectStage/{workId}", method = RequestMethod.GET)
    public List<MarkSubject> getAllSubjectStage(@PathVariable Long workId) {
        return markSubjectRepo.findAllByWorkId(workId);
    }

    /**
     * 当个评卷科目信息
     *
     * @param markSubjectId 评卷科目id
     * @return
     */
    @RequestMapping(value = "{markSubjectId}", method = RequestMethod.GET)
    public MarkSubject get(@PathVariable String markSubjectId) {
        MarkSubject subject = markSubjectRepo.findOne(markSubjectId);
        return subject;
    }

    /**
     * 该评卷科目下的评卷员列表
     *
     * @param markSubject 评卷科目id
     * @return
     */
    @RequestMapping(value = "{markSubject}/markers", method = RequestMethod.GET)
    public List<MarkUser> markUsers(@PathVariable MarkSubject markSubject) {
        List<MarkUser> markUsers = markUserRepo.findByWorkIdAndSubjectAndRole(markSubject.getWorkId(), markSubject.getSubject(), Role.MARKER);
        if (MarkStage.LEVEL == markSubject.getStage()) {
            markUsers = markUsers.stream().filter(m -> m.getMarkRight() == MarkRight.ALLOW_LEVELING || m.getMarkRight() == MarkRight.ALLOW_ALL).collect(Collectors.toList());
        } else if (MarkStage.SCORE == markSubject.getStage()) {
            markUsers = markUsers.stream().filter(m -> m.getMarkRight() == MarkRight.ALLOW_SCORING || m.getMarkRight() == MarkRight.ALLOW_ALL).collect(Collectors.toList());
        }
        return markUsers;
    }

    /**
     * 该评卷科目的试题统计信息
     *
     * @param markSubject 评卷科目id
     * @return
     */
    @RequestMapping(value = "{markSubject}/stat/questions", method = RequestMethod.GET)
    public List<QuestionStatDTO> questions(@PathVariable MarkSubject markSubject) {
        List<QuestionStatDTO> questionStatDTOs = new ArrayList<>();
        List<Object[]> qStats = null;
        if (markSubject.getStage() == MarkStage.LEVEL) {
            qStats = paperRepo.countGroupByQuestionForLevel(markSubject.getWorkId(), markSubject.getSubject().toString(), markSubject.getTest());
        } else {
            qStats = paperRepo.countGroupByQuestionForScore(markSubject.getWorkId(), markSubject.getSubject().toString());
        }
        for (Object[] objects : qStats) {
            QuestionStatDTO dto = questionStatAssembler.toDTO(objects);
            questionStatDTOs.add(dto);
        }
        return questionStatDTOs;
    }

    /**
     * 该评卷科目的分档统计信息
     *
     * @param markSubject 评卷科目id
     * @param questionId  试题id
     * @return
     */
    @RequestMapping(value = "{markSubject}/stat/levels", method = RequestMethod.GET)
    public List<LevelStatDTO> levelStat(HttpServletRequest request, @PathVariable MarkSubject markSubject, @RequestParam Long questionId) {
        List<LevelStatDTO> levelStatDTOs = new ArrayList<>();
        paperRepo.countGroupByLevelAll(questionId, markSubject.getTest())
                .forEach(o -> {
                    levelStatDTOs.add(levelStatAssembler.toDTO(o));
                });
        //2019-11-12备用
        /*paperRepo.countGroupByLevelAllOther(questionId, questionId, questionId)
                .forEach(o -> {
                    levelStatDTOs.add(levelStatAssembler.toDTO(o));
                });*/
        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
        for (Level level : levels) {
            long count = levelStatDTOs.stream().filter(l -> String.valueOf(l.getId()).equals(level.getCode())).count();
            if (count == 0) {
                LevelStatDTO dto = new LevelStatDTO();
                dto.setId(level.getCode());
                dto.setCount(0);
                dto.setPercent(0.0);
                levelStatDTOs.add(dto);
            }
        }
        long total = paperRepo.countByWorkIdAndQuestionId(markSubject.getWorkId(), questionId);
        levelStatDTOs.forEach(o -> {
            if (o.getId() != null) {
                double p = (double) o.getCount() / total;
                BigDecimal bd = new BigDecimal(p).setScale(3, RoundingMode.HALF_EVEN);
                o.setPercent(bd.doubleValue());
            }
        });
        return levelStatDTOs;
    }

    /**
     * 该评卷科目下的评卷员分组列表
     *
     * @param markSubject 评卷科目id
     * @return
     */
    @RequestMapping(value = "{markSubject}/markergroups", method = RequestMethod.GET)
    public List<MarkerGroup> getMarkerGroups(@PathVariable MarkSubject markSubject) {
        return markerGroupRepo.findByWorkIdAndSubject(markSubject.getWorkId(), markSubject.getSubject());
    }

    /**
     * 创建评卷组
     *
     * @param markSubject 评卷科目id
     * @param markerGroup 评卷组
     */
    @RequestMapping(value = "{markSubject}/markergroups", method = RequestMethod.POST)
    public MarkerGroup createMarkerGroup(@PathVariable MarkSubject markSubject, @RequestBody MarkerGroup markerGroup) {
        markerGroup.setWorkId(markSubject.getWorkId());
        markerGroup.setSubject(markSubject.getSubject());
        return markerGroupRepo.save(markerGroup);
    }

    /**
     * 修改评卷组名
     *
     * @param markSubject 评卷科目id
     * @param domain      评卷组id
     * @param markerGroup 评卷组
     */
    @RequestMapping(value = "{markSubject}/markergroups/{domain}", method = RequestMethod.PATCH)
    public void updateMarkerGroup(@PathVariable MarkSubject markSubject, @PathVariable MarkerGroup domain, @RequestBody MarkerGroup markerGroup) {
        domain.setName(markerGroup.getName());
        markerGroupRepo.save(markerGroup);
    }

    /**
     * 删除评卷组
     *
     * @param markSubject 评卷科目id
     * @param domain      评卷组id
     */
    @RequestMapping(value = "{markSubject}/markergroups/{domain}", method = RequestMethod.DELETE)
    public void removeMarkerGroup(@PathVariable MarkSubject markSubject, @PathVariable MarkerGroup domain) {
        markerGroupRepo.delete(domain);
    }

    /**
     * 单个评卷组信息
     *
     * @param markSubject 评卷科目id
     * @param domain      评卷组id
     * @return
     */
    @RequestMapping(value = "{markSubject}/markergroups/{domain}", method = RequestMethod.GET)
    public MarkerGroup getMarkerGroup(@PathVariable MarkSubject markSubject, @PathVariable MarkerGroup domain) {
        return domain;
    }

    /**
     * 为评卷组设置评卷员
     *
     * @param markSubject 评卷科目id
     * @param domain      评卷组id
     * @param markerIds   评卷员用户id数组
     */
    @RequestMapping(value = "{markSubject}/markergroups/{domain}/setmaker", method = RequestMethod.PATCH)
    public void setMaker(@PathVariable MarkSubject markSubject, @PathVariable MarkerGroup domain, @RequestParam Long... markerIds) {
        domain.setMarkers(new HashSet<MarkUser>());
        for (Long markerId : markerIds) {
            domain.addMarker(markUserRepo.findOne(markerId));
        }
        markerGroupRepo.save(domain);
    }

    /**
     * 该评卷科目进入下一阶段
     *
     * @param markSubject 评卷科目id
     * @param taskCount   分配任务数量
     */
    @RequestMapping(value = "{markSubject}", method = RequestMethod.PATCH)
    public void goNext(@PathVariable MarkSubject markSubject, @RequestParam(value = "taskCount", defaultValue = "0") int taskCount) throws Exception {
        stageControlService.goNext(markSubject, taskCount);
    }

    /**
     * 查询分档进度
     *
     * @param markSubject 评卷科目id
     */
    @RequestMapping(value = "{markSubject}/progress", method = RequestMethod.GET)
    public LevleProgressDTO progress(@PathVariable MarkSubject markSubject) {
        return stageControlService.progress(markSubject);
    }
}
