package cn.com.qmth.stmms.ms.marking.assembler;

import cn.com.qmth.stmms.ms.commons.config.ImageConfig;
import cn.com.qmth.stmms.ms.commons.config.SystemConfig;
import cn.com.qmth.stmms.ms.commons.constant.SystemConstant;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.Student;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.repository.MarkSubjectRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkTaskRepo;
import cn.com.qmth.stmms.ms.core.repository.StudentRepo;
import cn.com.qmth.stmms.ms.marking.dto.PaperDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * Created by zhengmin on 2016/10/10.
 */
@Component
public class PaperAssembler {

    @Autowired
    private MarkTaskRepo markTaskRepo;

    @Autowired
    private MarkTaskAssembler markTaskAssembler;

    @Autowired
    private SystemConfig systemConfig;

    @Autowired
    ImageConfig imageConfig;

    @Autowired
    StudentRepo studentRepo;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    public PaperDTO toDTO(Paper paper) {
        String imgSrc = null;
        String thumbSrc = null;
        if (imageConfig.isImageEnc()) {
            Student student = studentRepo.findByWorkIdAndExamNumberAndTest(paper.getWorkId(), paper.getExamNumber(), String.valueOf(TrialEnum.DEFAULT.getId()));
            imgSrc = systemConfig.getImageUrl(paper.getWorkId(), paper.getSubject().ordinal(), student.getId(), SystemConstant.IMAGE);
            thumbSrc = systemConfig.getThumbUrl(paper.getWorkId(), paper.getSubject().ordinal(), student.getId(), SystemConstant.THUMB);
        } else {
            imgSrc = systemConfig.getImageUrl(paper.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
            thumbSrc = systemConfig.getThumbUrl(paper.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
        }
        //查询阶段
        MarkSubject markSubject = markSubjectRepo.findOne(paper.getWorkId() + "-" + paper.getSubject());
        PaperDTO paperDTO = null;
        if (paper != null) {
            paperDTO = new PaperDTO();
            paperDTO.setId(paper.getId());
            paperDTO.setExamNumber(paper.getExamNumber());
            paperDTO.setSn(paper.getSecretNumber());
            paperDTO.setLevel(paper.getLevel());
            paperDTO.setManual(paper.isManual());
            paperDTO.setScore(paper.getScore());
            paperDTO.setArbitrated(paper.isArbitrated());
            paperDTO.setRejected(paper.isRejected());
            paperDTO.setSample(paper.isSample());
            paperDTO.setMarkByLeader(paper.isMarkByLeader());
            paperDTO.setTagged(paper.isTagged());
            paperDTO.setMarkedLogic(paper.isMarkedLogic());
            paperDTO.setRedoLevel(paper.getRedoLevel());
            paperDTO.setUpdatedOn(paper.getUpdatedOn());
//            String imgSrc = systemConfig.getImageUrl(paper.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
//            String thumbSrc = systemConfig.getThumbUrl(paper.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
            paperDTO.setImgSrc(imgSrc);
            paperDTO.setThumbSrc(thumbSrc);
            paperDTO.setAreaCode(paper.getAreaCode());
            paperDTO.setMissing(paper.getIsMissing());
            if (paper.getInspectScore() != null) {
                paperDTO.setInspectScore(paper.getInspectScore().intValue());
            }
            if (paper.getInspectLevel() != null) {
                paperDTO.setInspectLevel(paper.getInspectLevel());
            }
            paperDTO.setStage(markSubject.getStage());
            paperDTO.setTest(markSubject.getTest());
            paperDTO.setPaperTest(paper.getTest());
        }
        return paperDTO;
    }

    public PaperDTO toDTOForSheet(Paper paper) {
        String imgSrc = null;
        String thumbSrc = null;
        String sheetSrc = null;
        if (imageConfig.isImageEnc()) {
            Student student = studentRepo.findByWorkIdAndExamNumberAndTest(paper.getWorkId(), paper.getExamNumber(), String.valueOf(TrialEnum.DEFAULT.getId()));
            imgSrc = systemConfig.getImageUrl(paper.getWorkId(), paper.getSubject().ordinal(), student.getId(), SystemConstant.IMAGE);
            thumbSrc = systemConfig.getThumbUrl(paper.getWorkId(), paper.getSubject().ordinal(), student.getId(), SystemConstant.THUMB);
            sheetSrc = systemConfig.getSheetUrl(paper.getWorkId(), paper.getSubject().ordinal(), student.getId(), SystemConstant.SHEET);
        } else {
            imgSrc = systemConfig.getImageUrl(paper.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
            thumbSrc = systemConfig.getThumbUrl(paper.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
            sheetSrc = systemConfig.getSheetUrl(paper.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
        }
        //查询阶段
        MarkSubject markSubject = markSubjectRepo.findOne(paper.getWorkId() + "-" + paper.getSubject());
        PaperDTO paperDTO = null;
        if (paper != null) {
            paperDTO = new PaperDTO();
            paperDTO.setId(paper.getId());
            paperDTO.setExamNumber(paper.getExamNumber());
            paperDTO.setSn(paper.getSecretNumber());
            paperDTO.setLevel(paper.getLevel());
            paperDTO.setManual(paper.isManual());
            paperDTO.setScore(paper.getScore());
            paperDTO.setArbitrated(paper.isArbitrated());
            paperDTO.setRejected(paper.isRejected());
            paperDTO.setSample(paper.isSample());
            paperDTO.setMarkByLeader(paper.isMarkByLeader());
            paperDTO.setTagged(paper.isTagged());
            paperDTO.setMarkedLogic(paper.isMarkedLogic());
            paperDTO.setRedoLevel(paper.getRedoLevel());
            paperDTO.setUpdatedOn(paper.getUpdatedOn());
//            String imgSrc = systemConfig.getImageUrl(paper.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
//            String thumbSrc = systemConfig.getThumbUrl(paper.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
//            String sheetSrc = systemConfig.getSheetUrl(paper.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
            paperDTO.setImgSrc(imgSrc);
            paperDTO.setThumbSrc(thumbSrc);
            paperDTO.setSheetSrc(sheetSrc);
            paperDTO.setAreaCode(paper.getAreaCode());
            if (paper.getInspectScore() != null) {
                paperDTO.setInspectScore(paper.getInspectScore().intValue());
            }
            if (paper.getInspectLevel() != null) {
                paperDTO.setInspectLevel(paper.getInspectLevel());
            }
            paperDTO.setMissing(paper.getIsMissing());
            paperDTO.setStage(markSubject.getStage());
            paperDTO.setTest(paper.getTest());
            paperDTO.setPaperTest(paper.getTest());
        }
        return paperDTO;
    }
}
